########
# PROGRAM FOR CONVERTING REAL SPACE VECTORS TO RECIPROCAL SPACE VECTORS
# MATTHEW SRNEC and SHIV UPADHYAY
########
import scipy as scp
import numpy as np
import sympy as syp
from sympy import *
from sympy import init_printing;
init_printing(use_latex = 'mathjax')
from sympy import pprint
import re
from colorama import Fore, Back, Style
from colorama import init
init(autoreset=True)
########
print '\n'
print 'Welcome to the Reciprocal Space Vector Calculator. This program converts real space translation vectors into reciprocal space vectors.'
print '\n'
print 'Created by Matthew Srnec and Shiv Upadhyay'
print '\n'
print 'Please enter the case number you would like to study.'
print '\n'
print 'Cases:'
print '\t 1. Two-dimensional system.'
print '\t 2. Three-dimensional system.'
print '\t 99. Quit.'
print '\n'
Case=0
while(Case not in [1,2,99]):
	try:
		Case=input('Enter case number (1,2, or 99): ')
	except:
		Case=0
	if(Case not in [1,2,99]):
		print '\n'
		print Fore.RED+'Please enter a valid input!'
		print '\n'
while(Case!=99 and Case in [1,2]):
	#### Two-dimensional case
	if(Case==1):
		print '\n'
		print 'Considering a 2-dimensional case, recall that the lattice parameters "a" and "b" are equivalent for the cubic, tetragonal, hexagonal, and rhombohedral crystal systems. For the orthorhombic, monoclinic, and triclinic crystal systems, the "a" and "b" lattice parameters are unequal.  Your inputs for the x and y coordinates of the real space translation vectors, r1 and r2, should reflect this.'
		print '\n'
		#### User input for x and y coordinates of real space vectors
		x1=raw_input('Please enter the x-coordinate of real space translation vector, r1. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		x1_sym=syp.sympify(x1)
		print '\n'
		y1=raw_input('Please enter the y-coordinate of real space translation vector, r1. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		y1_sym=syp.sympify(y1)
		r1=scp.array([x1_sym,y1_sym])
		print '\n'
		print 'r1 = ',r1
		print '\n'
		x2=raw_input('Please enter the x-coordinate of real space translation vector, r2. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		x2_sym=syp.sympify(x2)
		print '\n'
		y2=raw_input('Please enter the y-coordinate of real space translation vector, r2. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		y2_sym=syp.sympify(y2)
		r2=scp.array([x2_sym,y2_sym])
		print '\n'
		print 'r2 = ',r2
		print '\n'
		#### User input of rotational matrix
		while(True):
			try:
				angle=float(raw_input('Please enter the angle (degrees) for the rotational matrix.  Clockwise rotations have negative values while counterclockwise rotations have positive values. '))
			except ValueError:
				print '\n'
				print Fore.RED+'Please enter a valid input!'
				print '\n'
				continue
			theta=(angle/180.)*scp.pi
			gamma=scp.array([[int(scp.cos(theta)),int(-scp.sin(theta))],[int(scp.sin(theta)),int(scp.cos(theta))]])
			print '\n'
			print 'gamma =',gamma
			print '\n'
			phase1=scp.array([2*syp.pi,0])
			phase2=scp.array([0,2*syp.pi])
			#### Determination of reciprocal space vectors
			k1=scp.dot(phase1,scp.dot(gamma,r2.T))/scp.dot(r1,scp.dot(gamma,r2.T)),scp.dot(phase2,scp.dot(gamma,r2.T))/scp.dot(r1,scp.dot(gamma,r2.T))
			print '\n'
			print 'k1 = ';pprint(k1)
			k2=scp.dot(phase1,scp.dot(gamma.T,r1.T))/scp.dot(r1,scp.dot(gamma,r2.T)),scp.dot(phase2,scp.dot(gamma.T,r1.T))/scp.dot(r1,scp.dot(gamma,r2.T))
			print '\n'
			print 'k2 = ';pprint(k2)
			#### Validation of reciprocal space vectors using dot products
			print '\n'
			print 'r1 dot k1 = ';pprint(scp.dot(r1,k1))
			print '\n'
			print 'r2 dot k2 = ';pprint(scp.dot(r2,k2))
			print '\n'
			print 'r1 dot k2 = ';pprint(scp.dot(r1,k2))
			print '\n'
			print 'r2 dot k1 = ';pprint(scp.dot(r2,k1))
        		break
	#### Three-dimensional case
	elif(Case==2):
		print '\n'
		print 'Considering a 3-dimensional case, recall that the lattice parameters "a", "b", and "c" are equivalent for the cubic and trigonal crystal systems.  In the tetragonal and hexagonal crystal systems, "a" and "b" are equal, but are unequal to "c". For the orthorhombic, monoclinic, and triclinic crystal systems, the "a", "b", and "c" lattice parameters are all unequal.  Your inputs for the x and y coordinates of the real space translation vectors, r1 and r2, should reflect this.'
		#### User input for x and y coordinates of real space vectors
		print '\n'
		x1=raw_input('Please enter the x-coordinate of real space translation vector, r1. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		x1_sym=syp.sympify(x1)
		print '\n'
		y1=raw_input('Please enter the y-coordinate of real space translation vector, r1. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		y1_sym=syp.sympify(y1)
		print '\n'
		z1=raw_input('Please enter the z-coordinate of real space translation vector, r1. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		z1_sym=syp.sympify(z1)
		r1=scp.array([x1_sym,y1_sym,z1_sym])
		print '\n'
		print 'r1 = ',r1
		print '\n'
		x2=raw_input('Please enter the x-coordinate of real space translation vector, r2. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		x2_sym=syp.sympify(x2)
		print '\n'
		y2=raw_input('Please enter the y-coordinate of real space translation vector, r2. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		y2_sym=syp.sympify(y2)
		print '\n'
		z2=raw_input('Please enter the z-coordinate of real space translation vector, r2. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		z2_sym=syp.sympify(z2)
		r2=scp.array([x2_sym,y2_sym,z2_sym])
		print '\n'
		print 'r2 = ',r2
		print '\n'
		x3=raw_input('Please enter the x-coordinate of real space translation vector, r3. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		x3_sym=syp.sympify(x3)
		print '\n'
		y3=raw_input('Please enter the y-coordinate of real space translation vector, r3. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		y3_sym=syp.sympify(y3)
		print '\n'
		z3=raw_input('Please enter the z-coordinate of real space translation vector, r3. If your input contains multiplication, enter this using the symbol *. For example, 3*a. ')
		z3_sym=syp.sympify(z3)
		r3=scp.array([x3_sym,y3_sym,z3_sym])
		print '\n'
		print 'r3 = ',r3
    		#### Determination of reciprocal space vectors
		print '\n'
		phase1=scp.array([2*syp.pi,0,0])
		phase2=scp.array([0,2*syp.pi,0])
		phase3=scp.array([0,0,2*syp.pi])
		k1=scp.dot(phase1,scp.cross(r2,r3))/(scp.dot(r1,scp.cross(r2,r3))),scp.dot(phase2,scp.cross(r2,r3))/(scp.dot(r1,scp.cross(r2,r3))),scp.dot(phase3,scp.cross(r2,r3))/(scp.dot(r1,scp.cross(r2,r3)))
		print '\n'
		print 'k1 = ';pprint(k1)
		k2=scp.dot(phase1,scp.cross(r1,r3))/(scp.dot(r2,scp.cross(r1,r3))),scp.dot(phase2,scp.cross(r1,r3))/(scp.dot(r2,scp.cross(r1,r3))),scp.dot(phase3,scp.cross(r1,r3))/(scp.dot(r2,scp.cross(r1,r3)))
		print '\n'
		print 'k2 = ';pprint(k2)
		k3=scp.dot(phase1,scp.cross(r1,r2))/(scp.dot(r3,scp.cross(r1,r2))),scp.dot(phase2,scp.cross(r1,r2))/(scp.dot(r3,scp.cross(r1,r2))),scp.dot(phase3,scp.cross(r1,r2))/(scp.dot(r3,scp.cross(r1,r2)))
		print '\n'
		print 'k3 = ';pprint(k3)
		#### Validation of reciprocal space vectors using dot products
		print '\n'
		print 'r1 dot k1 = ';pprint(scp.dot(r1,k1))
		print '\n'
		print 'r2 dot k2 = ';pprint(scp.dot(r2,k2))
		print '\n'
		print 'r3 dot k3 = ';pprint(scp.dot(r3,k3))
		print '\n'
		print 'r1 dot k2 = ';pprint(scp.dot(r1,k2))
		print '\n'
		print 'r2 dot k1 = ';pprint(scp.dot(r2,k1))
		print '\n'
		print 'r1 dot k3 = ';pprint(scp.dot(r1,k3))
		print '\n'
		print 'r3 dot k1 = ';pprint(scp.dot(r3,k1))
		print '\n'
		print 'r2 dot k3 = ';pprint(scp.dot(r2,k3))
		print '\n'
		print 'r3 dot k2 = ';pprint(scp.dot(r3,k2))      
		print '\n'
########
	elif(Case==99):
        	exit()
	print "\n"
	print 'Please enter another case to study.'
	print "\n"
	print 'Cases:'
	print '\t 1. Two-dimensional system.'
	print '\t 2. Three-dimensional system.'
	print '\t 99. Quit.'
	print "\n"
	Case=0
	while(Case not in [1,2,99]):
        	try:
			Case=input('Enter case number (1,2, or 99): ')
		except:
			Case=0
		if(Case not in [1,2,99]):
			print '\n'
			print Fore.RED+'Please enter a valid input!'
			print '\n'
######## THE END
